/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/


var initComplete = false;
var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

        },1);

	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}

// Below is a simple example of Parent and Child browser windows and how they communicate.
// The idea is to use window.open(url, ...) to open a child browser window.
//
// For communication, the idea is to add specific object/APIs to the opened child window.
// In particular, the parent add its own object/APIs to the child window so that the
// child can invoke parent's APIs.
// The child (see child.html) also add its own specific object/APIs to the
// child window itself so that the parent can use those child APIs.
// Another option for communication is to use Window.postMessage. A good example 
// is given in https://developer.mozilla.org/en-US/docs/Web/API/Window/postMessage.
//
// To develop app/child.html, you can hand-write the code.
// Alteratively, you use GC designer to develop a separate GC app (say childGUI) ,
// run the childGUI app to generate the content of its own index.html,
// and copy the content of that index.html to the app/child.html of this main GC app.
//
// Note about data binding -
// While you can get data binding working inside parent window, e.g.
// gc.databind.registry.getBinding('ti_widget_textbox1.value'), or 
// gc.databind.registry.bind('ti_widget_textbox1.value', 'ti_widget_textbox2.value'), etc.
// you cannot get data binding working inside child window or bind a widget in
// parnent to a widget in child.
// If you need this 'cross window binding', please let us know.

var fun1 = function (summary) {
    templateObj.$.ti_widget_textbox2.value = ' Receiving from Child... Min '+summary[0] + ', Max ' + summary[1] + ', Avg ' + summary[2] +
      '  and shared object becomes ' + JSON.stringify(childWindow.ppcMain.sharedObject);
}

var childWindow;
var onOpenChild = function() {
    if (childWindow === undefined) {
        // Open a child window for a specific url, here is an url under this gc app.
        childWindow = window.open('app/child.html', 'ChildGUI');
        // add specific object for communication
        childWindow.ppcMain = {};
        childWindow.ppcMain.mainWindow = window;
        childWindow.ppcMain.fun1 = fun1; // child window can call parent's fun1
        childWindow.ppcMain.sharedObject = {}; // shared object used by main and child
        childWindow.ppcMain.gc = gc;
    }
}
var onSend = function() {
    if (childWindow === undefined) {
        alert('Open Child GUI first and try to send again');
        return;
    }
    var x = [];
    for (var i=0; i<5;i++) {x.push(Math.round(Math.random()*100));}
    templateObj.$.ti_widget_textbox1.value = "Sending to Child ... " + x + '\n';
    childWindow.ppcMain.sharedObject.array1 = [11,12]; // also change ppcMain.sharedObj
    childWindow.ppcChild.fun2(x);
}



